@@ -32,6 +32,7 @@
 #include <regex.h>
 #include <limits.h>
 #include <ctype.h>
+#include <unistd.h>
 
 #include <string.h>
 
@@ -351,7 +352,17 @@ extract_secnum_from_filename (const char *filename)
 {
 	char *end_string;
 
+#ifdef HAVE_LIBBZ2
+	end_string = strstr (filename, ".bz2");
+	if (end_string) 
+		/* Its a bzipped man file so we need to return the
+		 * secnum */
+		return g_strndup (end_string - 3, 3);
+	else
+		free(end_string);
+#endif
 	end_string = strstr (filename, ".gz");
+
 	if (!end_string) {
 		/* Not a gzipped man file */
 		return g_strdup (strrchr (filename, '.'));
@@ -369,6 +380,15 @@ man_name_without_suffix (const char *filename)
 {
 	char *end_string, *new_string, *ptr;
 
+#ifdef HAVE_LIBBZ2
+	end_string = strstr (filename, ".bz2");
+	if (end_string) 
+		/* e.g. manfile.1.bz2  would return manfile */
+		return g_strndup (filename, end_string - filename - 3);
+	else
+		free(end_string);
+#endif
+
 	end_string = strstr (filename, ".gz");
 	if (end_string) {
 		/* e.g. manfile.1.gz  would return manfile */
@@ -420,6 +440,12 @@ fmt_man_populate_tree_for_subdir (HyperbolaDocTree *tree,
 
 		manname = man_name_without_suffix (dent->d_name);
 
+#ifdef HAVE_LIBBZ2
+		if (strstr (dent->d_name, ".bz2")) {
+			g_snprintf (namebuf, sizeof (namebuf), "%.*s",
+				    (int) strlen (manname), manname);
+		} else
+#endif
 		if (strstr (dent->d_name, ".gz")) {
 			g_snprintf (namebuf, sizeof (namebuf), "%.*s",
 				    (int) strlen (manname), manname);
@@ -501,21 +527,20 @@ translate_array (char **array)
                       working on changes to the kernel)
 ***/
 
-static char *man_path[] = { N_("Manual"), NULL };
-static char *cfg_path[] =
-	{ N_("Manual"), N_("System"), N_("Configuration"), N_("Config files"),
-	NULL
-};
-static char *app_path[] =
-	{ N_("Manual"), N_("Applications"), N_("Command Line"), NULL };
-static char *dev_path[] =
-	{ N_("Manual"), N_("Development"), N_("APIs"), N_("Miscellaneous"),
-	NULL
-};
+static char *app_path[] = { N_("Manual"), N_("Applications"), NULL };
 static char *syscall_path[] =
-	{ N_("Manual"), N_("Development"), N_("APIs"), N_("System Calls"),
-	NULL
-};
+	{ N_("Manual"), N_("Development"), N_("System Calls"), NULL };
+static char *libs_path[] =
+	{ N_("Manual"), N_("Development"), N_("Library Functions"), NULL };
+static char *devs_path[] =
+	{ N_("Manual"), N_("Development"), N_("Devices"), NULL };
+static char *cfg_path[] = { N_("Manual"), N_("Configuration Files"), NULL };
+static char *games_path[] = { N_("Manual"), N_("Games"), NULL };
+static char *convs_path[] = { N_("Manual"), N_("Conventions"), NULL };
+static char *sysadm_path[] =
+	{ N_("Manual"), N_("System Administration"), NULL };
+static char *kern_path[] =
+	{ N_("Manual"), N_("Development"), N_("Kernel Routines"), NULL };
 
 static void
 fmt_man_populate_tree_for_dir (HyperbolaDocTree * tree, const char *basedir)
@@ -529,25 +554,25 @@ fmt_man_populate_tree_for_dir (HyperbolaDocTree * tree, const char *basedir)
 	fmt_man_populate_tree_for_subdir (tree, cbuf, syscall_path, '2');
 
 	g_snprintf (cbuf, sizeof (cbuf), "%s/man3", basedir);
-	fmt_man_populate_tree_for_subdir (tree, cbuf, dev_path, '3');
+	fmt_man_populate_tree_for_subdir (tree, cbuf, libs_path, '3');
 
 	g_snprintf (cbuf, sizeof (cbuf), "%s/man4", basedir);
-	fmt_man_populate_tree_for_subdir (tree, cbuf, man_path, '4');
+	fmt_man_populate_tree_for_subdir (tree, cbuf, devs_path, '4');
 
 	g_snprintf (cbuf, sizeof (cbuf), "%s/man5", basedir);
 	fmt_man_populate_tree_for_subdir (tree, cbuf, cfg_path, '5');
 
 	g_snprintf (cbuf, sizeof (cbuf), "%s/man6", basedir);
-	fmt_man_populate_tree_for_subdir (tree, cbuf, app_path, '6');
+	fmt_man_populate_tree_for_subdir (tree, cbuf, games_path, '6');
 
 	g_snprintf (cbuf, sizeof (cbuf), "%s/man7", basedir);
-	fmt_man_populate_tree_for_subdir (tree, cbuf, man_path, '7');
+	fmt_man_populate_tree_for_subdir (tree, cbuf, convs_path, '7');
 
 	g_snprintf (cbuf, sizeof (cbuf), "%s/man8", basedir);
-	fmt_man_populate_tree_for_subdir (tree, cbuf, app_path, '8');
+	fmt_man_populate_tree_for_subdir (tree, cbuf, sysadm_path, '8');
 
 	g_snprintf (cbuf, sizeof (cbuf), "%s/man9", basedir);
-	fmt_man_populate_tree_for_subdir (tree, cbuf, man_path, '9');
+	fmt_man_populate_tree_for_subdir (tree, cbuf, kern_path, '9');
 }
 
 static void
@@ -574,23 +599,25 @@ fmt_man_populate_tree (HyperbolaDocTree * tree)
 	char aline[1024];
 	char **manpath = NULL;
 	int i;
+	char ***p;
+	char **paths[] = {
+		app_path, syscall_path, libs_path, devs_path, cfg_path,
+		games_path, convs_path, sysadm_path, kern_path, NULL
+	};
+
 	/* Go through all the man pages:
 	 * 1. Determine the places to search (run 'manpath').
 	 * 2. Go through all subdirectories to find individual files.
 	 * 3. For each file, add it onto the tree at the right place.
 	 */
 
-	translate_array (man_path);
-	translate_array (cfg_path);
-	translate_array (app_path);
-	translate_array (dev_path);
-	translate_array (syscall_path);
+	for (p = paths; *p; p++) {
+		translate_array (*p);
+	}
 
-	make_treesection (tree, man_path);
-	make_treesection (tree, cfg_path);
-	make_treesection (tree, app_path);
-	make_treesection (tree, dev_path);
-	make_treesection (tree, syscall_path);
+	for (p = paths; *p; p++) {
+		make_treesection (tree, *p);
+	}
 
 	fh = popen ("manpath", "r");
 	g_return_if_fail (fh);
@@ -1044,8 +1071,10 @@ fmt_scrollkeeper_parse_doc_toc (HyperbolaDocTree * tree, char **ancestors,
     	xmlDefaultSAXHandler.fatalError = xml_fatal_error_handler;
 
 	if (!toc_doc) {
+		/*
 		g_warning ("Unable to parse ScrollKeeper TOC XML file:\n\t%s",
 			   toc_file);
+		*/
 		return;
 	}
 
@@ -1385,20 +1414,24 @@ static void
 fmt_scrollkeeper_populate_tree (HyperbolaDocTree * tree)
 {
 	xmlDocPtr doc;
-	char *locale;
+	GList *node;
 	
 	char *tree_path[] = { NULL };
 
-	locale = getenv ("LANG");
-	if (locale == NULL)
-	    return;
-
-	doc = fmt_scrollkeeper_get_xml_tree_of_locale (locale);
-	if (strcmp(locale, "C") &&
-	    (doc == NULL ||
-	     (doc->root != NULL &&
-	      fmt_scrollkeeper_tree_empty(doc->root->childs))))
-		doc = fmt_scrollkeeper_get_xml_tree_of_locale("C");
+	doc = NULL;
+	for (node = gnome_i18n_get_language_list ("LC_MESSAGES");
+	     node != NULL;
+	     node = node->next) {
+		doc = fmt_scrollkeeper_get_xml_tree_of_locale (node->data);
+		if (doc != NULL) {
+			if (doc->root != NULL && !fmt_scrollkeeper_tree_empty(doc->root->childs)) {
+				break;
+			} else {
+				xmlFreeDoc (doc);
+				doc = NULL;
+			}
+		}
+	}
 		
 	if (doc) {
 		fmt_scrollkeeper_trim_empty_branches (doc->root->childs);
@@ -1467,7 +1500,7 @@ static void fmt_toplevel_add_doc (HyperbolaDocTree * tree, char * omf_name)
 
 	    		char *ptr;
 	    
-	    		ptr = strdup(node->childs->content);
+	    		ptr = g_strdup(node->childs->content);
 	    		str = remove_leading_and_trailing_white_spaces(ptr);
 	    		title = g_strconcat(prefix, str, NULL);
 	    		g_free(ptr);	
@@ -1512,20 +1545,95 @@ static void fmt_toplevel_add_doc (HyperbolaDocTree * tree, char * omf_name)
     	}
 }
 
-static void fmt_toplevel_parse_xml_tree (HyperbolaDocTree * tree, xmlDocPtr doc,
-						char *locale)
+/* returns -1 on invalid locale (not found), or the position
+ * in the locale list.  the lower the position the better, that
+ * is, the lower number should get precedence */
+static int
+locale_score (GList *locales, xmlNode *doc_node)
 {
-	xmlNodePtr doc_node, docpath_node;
-    	char *omf_name, *str, *doc_locale, omf_dir[256], omf_path[256];
+	GList *li;
+	char *locale;
+	int score;
+
+	if (doc_node == NULL)
+		return -1;
+
+	locale = xmlGetProp (doc_node, "locale");
+	if (locale == NULL) {
+		locale = "C";
+	}
+
+	score = 0;
+	for (li = locales; li != NULL; li = li->next) {
+		if (strcmp (locale, li->data) == 0) {
+			return score;
+		}
+
+		score++;
+	}
+
+	return -1;
+}
+
+/* do we want to use new_doc rather then current_doc.  That is, is new_doc
+ * a better translation then current_doc */
+static gboolean
+is_new_locale_better (GList *locales, xmlNode *current_doc, xmlNode *new_doc)
+{
+	int current_score, new_score;
+
+	new_score = locale_score (locales, new_doc);
+
+	/* if new document is not in our list, forget it */
+	if (new_score == -1) {
+		return FALSE;
+	}
+
+	current_score = locale_score (locales, current_doc);
+
+	/* if new document is better (has lower score) then we want to use
+	 * that, or if the current document is bogus (-1) */
+	if (new_score < current_score
+	    || current_score == -1) {
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
+static char *
+get_path_from_node (const char *omf_dir, xmlNode *docpath_node)
+{
+	char *str, *omf_path, *omf_name;
+
+	str = g_strdup (docpath_node->childs->content);
+	omf_name = remove_leading_and_trailing_white_spaces (str);
+	omf_path = g_strdup_printf ("%s/%s", omf_dir, omf_name);
+
+	g_free (str);
+
+	return omf_path;
+}
+
+
+/* Note: Locales should include "C" as the last element.  Basically for use
+ * with locales lists returned by gnome_i18n_get_language_list. */
+static gboolean
+fmt_toplevel_parse_xml_tree (HyperbolaDocTree * tree,
+			     xmlDocPtr doc,
+			     GList *locales)
+{
+	xmlNodePtr doc_node, docpath_node, best_path_node;
+    	char omf_dir[256], *omf_path;
 	FILE *pipe;
 	int bytes_read;
-	int node_added;
+	gboolean node_added;
 
     	if (doc == NULL || doc->root == NULL)
-        	return;
+        	return FALSE;
 
-	if (locale == NULL)
-		return;
+	if (locales == NULL)
+		return FALSE;
 		
 	pipe = popen ("scrollkeeper-config --omfdir", "r");
 	bytes_read = fread ((void *) omf_dir, sizeof (char), 128, pipe);
@@ -1533,7 +1641,7 @@ static void fmt_toplevel_parse_xml_tree (HyperbolaDocTree * tree, xmlDocPtr doc,
 	/* Make sure that we don't end up out-of-bunds */
 	if (bytes_read < 1) {
 		pclose (pipe);
-		return;
+		return FALSE;
 	}
 
 	/* Make sure the string is properly terminated */
@@ -1541,57 +1649,48 @@ static void fmt_toplevel_parse_xml_tree (HyperbolaDocTree * tree, xmlDocPtr doc,
 
 	/* Exit code of 0 means we got a path back from ScrollKeeper */
 	if (pclose (pipe))
-		return;
+		return FALSE;
+
+	node_added = FALSE;
 	
     	for(doc_node = doc->root->childs; doc_node != NULL; 
             doc_node = doc_node->next) {
-	    
+
+		/* nothing found yet */
+		best_path_node = NULL;
+
 		/* check out the doc for the current locale */
 
-	        node_added = 0;
-	    
         	for(docpath_node = doc_node->childs; 
 	    	    docpath_node != NULL;
 	    	    docpath_node = docpath_node->next) {	
-		    
-	    		doc_locale = xmlGetProp(docpath_node, "locale");
-	    		if (doc_locale != NULL && !strcmp(doc_locale, locale)) {
-	        		if (docpath_node->childs != NULL && 
-	            		    docpath_node->childs->content != NULL) {
-				    
-	            			str = g_strdup(docpath_node->childs->content);
-		    			omf_name = remove_leading_and_trailing_white_spaces(str);
-					sprintf(omf_path, "%s/%s", omf_dir, omf_name);
-		    			fmt_toplevel_add_doc(tree, omf_path);
-		    			g_free(str);
-					node_added = 1;
-		 		}
-	    		}
-		}
-		
-		if (node_added || !strcmp(locale, "C"))
-			continue;
+			/* check validity of the node first */
+			if (docpath_node->childs != NULL
+			    && docpath_node->childs->content != NULL
+			    && is_new_locale_better (locales,
+						     best_path_node,
+						     docpath_node)) {
+				omf_path = get_path_from_node (omf_dir,
+							       docpath_node);
 
-		/* add the C locale one if the translated doc was not there */
-						
-		for(docpath_node = doc_node->childs; 
-	    	    docpath_node != NULL;
-	    	    docpath_node = docpath_node->next) {	
-		    
-		    	doc_locale = xmlGetProp(docpath_node, "locale");
-	    		if (doc_locale != NULL && !strcmp(doc_locale, "C")) {
-	        		if (docpath_node->childs != NULL && 
-	            		    docpath_node->childs->content != NULL) {
-				    
-	            			str = g_strdup(docpath_node->childs->content);
-		    			omf_name = remove_leading_and_trailing_white_spaces(str);
-					sprintf(omf_path, "%s/%s", omf_dir, omf_name);
-		    			fmt_toplevel_add_doc(tree, omf_path);
-		    			g_free(str);
-		 		}
-	    		}
+				if (access (omf_path, R_OK) == 0)
+					/* found a good one */
+					best_path_node = docpath_node;
+
+				g_free (omf_path);
+			}
+		}
+
+		if (best_path_node != NULL) {
+			omf_path = get_path_from_node (omf_dir, best_path_node);
+			fmt_toplevel_add_doc (tree, omf_path);
+			g_free (omf_path);
+
+			node_added = TRUE;
 		}
     	}
+
+	return node_added;
 }
 
 /* entry point for filling the toplevel tree that holds the very 
@@ -1605,28 +1704,22 @@ static int fmt_toplevel_populate_tree (HyperbolaDocTree * tree)
     	xmlDocPtr toplevel_doc;
     	char *toplevel_file;
 	int retval;
-	char *locale;
+	GList *locales;
   	
 	toplevel_file = HYPERBOLA_DATADIR "/topleveldocs.xml";
 
-	locale = getenv("LANG");
-	if (locale == NULL)
-		return 0;
-
     	toplevel_doc = xmlParseFile (toplevel_file); 
 	
-    	if (toplevel_doc) {
-        	fmt_toplevel_parse_xml_tree(tree, toplevel_doc, locale);
-		if (toplevel_doc->root != NULL &&
-		    toplevel_doc->root->childs != NULL &&
-		    toplevel_doc->root->childs->childs != NULL)
-		    	retval = 1;
-		else
-			retval = 0;
+    	if (toplevel_doc != NULL) {
+		locales = gnome_i18n_get_language_list ("LC_MESSAGES");
+		retval = fmt_toplevel_parse_xml_tree (tree, toplevel_doc,
+						      locales);
     	}
     	else {
 		retval = 0;
+		/*
         	g_warning ("Unable to locate toplevel XML file:\n\t%s", toplevel_file);
+		*/
     	}
 
     	xmlFreeDoc (toplevel_doc);
