@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 
 /*
- *  Copyright (C) 2000 Eazel, Inc.
+ *  Copyright (C) 2000, 2001 Eazel, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -101,25 +101,8 @@ struct NautilusMozillaContentViewDetails {
 						 * are not recorded in history
 						 */
 	gboolean	user_initiated_navigation;
+
 	BonoboUIComponent *ui;
-
-
-	/* For async NautilusView messages */
-	char		*pending_title;		/*for set_title*/
-	char		*pending_report_uri;	/*for report_location_change*/
-	char		*pending_link_message;	/*for report_status*/
-	double		pending_load_progress;	/*for load_progress*/
-	char		*pending_open_uri;	/*for open_location*/
-
-	gboolean	is_pending_set_title :1;
-	gboolean	is_pending_report_load_failed :1;
-	gboolean	is_pending_report_load_underway :1;
-	gboolean	is_pending_report_load_complete :1;
-	gboolean	is_pending_report_location_change :1;
-	gboolean	is_pending_report_status :1;
-	gboolean	is_pending_report_load_progress :1;
-	gboolean	is_pending_open_location :1;
-
 };
 
 /* GTK Type System */
@@ -141,12 +124,15 @@ static void	vfs_read_callback				(GnomeVFSAsyncHandle		*handle,
 								 gpointer			data);
 
 /* NautilusView callback functions */
-static void	view_load_location_callback			(NautilusView 			*nautilus_view, 
+static void	view_load_location_callback			(NautilusView 			*nautilus_view,
 								 const char 			*location,
 								 gpointer 			data);
 
 /* GtkEmbedMoz callback functions */
 
+static void	mozilla_realize_callback			(GtkWidget 			*mozilla,
+								 gpointer			user_data);
+
 static void	mozilla_title_changed_callback			(GtkMozEmbed 			*mozilla,
 								 gpointer			user_data);
 
@@ -233,34 +219,10 @@ static void	post_widget_initialize				(void);
 static char *	eazel_services_scheme_to_http			(NautilusMozillaContentView	*view,
 								 const char			*uri);
 
-static char *	eazel_services_scheme_from_http			(NautilusMozillaContentView	*view, 
+static char *	eazel_services_scheme_from_http			(NautilusMozillaContentView	*view,
 				  				 const char			*uri);
 #endif /* EAZEL_SERVICES */
 
-/*
- * Async NautilusView message dispatchers
- */
-
-static void	async_set_title 				(NautilusMozillaContentView 	*view,
-								 const char 			*title);
-static void	async_report_load_failed 			(NautilusMozillaContentView 	*view);
-
-static void	async_report_load_underway 			(NautilusMozillaContentView 	*view);
-
-static void	async_report_load_complete 			(NautilusMozillaContentView 	*view);
-
-static void 	async_report_location_change			(NautilusMozillaContentView	*view,
-								 const char 			*report_uri);
-static void 	async_report_status	 			(NautilusMozillaContentView	*view,
-								 const char 			*link_message);
-
-static void	async_report_load_progress 			(NautilusMozillaContentView 	*view,
-								 double 			load_progress);
-
-static void 	async_open_location	 			(NautilusMozillaContentView	*view,
-								 const char 			*open_uri);
-
-
 /* BonoboControl callbacks */
 static void bonobo_control_activate_callback (BonoboObject *control, gboolean state, gpointer callback_data);
 
@@ -326,6 +288,11 @@ nautilus_mozilla_content_view_initialize (NautilusMozillaContentView *view)
 	post_widget_initialize ();
 
 	/* Add callbacks to the beast */
+	gtk_signal_connect (GTK_OBJECT (view->details->mozilla), 
+				"realize",
+				GTK_SIGNAL_FUNC (mozilla_realize_callback),
+				view);
+
 	gtk_signal_connect (GTK_OBJECT (view->details->mozilla), 
 				"title",
 				GTK_SIGNAL_FUNC (mozilla_title_changed_callback),
@@ -482,7 +449,7 @@ try_transform_nautilus_uri_to_file_scheme (const char *uri)
 }
 
 static void
-view_load_location_callback (NautilusView *nautilus_view, 
+view_load_location_callback (NautilusView *nautilus_view,
 			     const char *location,
 			     gpointer data)
 {
@@ -493,8 +460,6 @@ view_load_location_callback (NautilusView *nautilus_view,
 
 	DEBUG_MSG (("+%s\n", __FUNCTION__));
 
-	DEBUG_MSG ((">nautilus_view_report_load_underway\n"));
-
 	/* set to FALSE here so any URI reported in mozilla_location_callback
 	 * will not be sent back to nautilus
 	 */
@@ -506,11 +471,13 @@ view_load_location_callback (NautilusView *nautilus_view,
 		/* if this is a gnome-help: uri, transform it into a file: uri
 		 * and load again to get around the fact that the help translater doesn't
 		 * know how to deal with paths such as "gnome-help:control-center/foo"
-		 * FIXME this results in an extraneous entry in the nautilus history
 		 */
-		nautilus_view_open_location_in_this_window (view->details->nautilus_view, file_uri);
+		DEBUG_MSG ((">nautilus_view_report_redirect (%s,%s)\n", location, file_uri));
+		nautilus_view_report_redirect (view->details->nautilus_view, location, file_uri, NULL, file_uri);
+		navigate_mozilla_to_nautilus_uri (view, file_uri);
 		g_free (file_uri);
 	} else {
+		DEBUG_MSG ((">nautilus_view_report_load_underway\n"));		
 		nautilus_view_report_load_underway (nautilus_view);
 		navigate_mozilla_to_nautilus_uri (view, location);
 	}
@@ -768,6 +735,29 @@ bonobo_control_activate_callback (BonoboObject *control, gboolean state, gpointe
 /***********************************************************************************/
 /***********************************************************************************/
 
+static void
+mozilla_realize_callback (GtkWidget *mozilla, gpointer user_data)
+{
+	NautilusMozillaContentView	*view;
+	char *uri;
+
+	DEBUG_MSG (("+%s\n", __FUNCTION__));
+
+	view = NAUTILUS_MOZILLA_CONTENT_VIEW (user_data);
+
+	g_assert (GTK_MOZ_EMBED (mozilla) == view->details->mozilla);
+
+	if (view->details->uri != NULL) {
+		DEBUG_MSG (("=%s navigating to uri after realize '%s'\n", __FUNCTION__, view->details->uri));
+
+		uri = g_strdup (view->details->uri);
+		navigate_mozilla_to_nautilus_uri (view, uri);
+		g_free (uri);
+	}
+
+	DEBUG_MSG (("-%s\n", __FUNCTION__));
+}
+
 static void
 mozilla_title_changed_callback (GtkMozEmbed *mozilla, gpointer user_data)
 {
@@ -785,7 +775,8 @@ mozilla_title_changed_callback (GtkMozEmbed *mozilla, gpointer user_data)
 	DEBUG_MSG (("=%s : new title='%s'\n", __FUNCTION__, new_title));
 
 	if (new_title != NULL && strcmp (new_title, "") != 0) {
-		async_set_title (view, new_title);
+		DEBUG_MSG ((">nautilus_view_set_title '%s'\n", new_title));
+		nautilus_view_set_title (view->details->nautilus_view, new_title);
 	}
 	
 	g_free (new_title);
@@ -844,7 +835,7 @@ mozilla_location_callback (GtkMozEmbed *mozilla, gpointer user_data)
 	DEBUG_MSG (("=%s : current='%s' new='%s'\n", __FUNCTION__, view->details->uri, new_location_translated));
 
 	/*
-	 * FIXME bug 6965
+	 * FIXME bug 7114
 	 * The user-initiated navigation check here is an attempt to be able
 	 * to prevent redirects that were handled by Mozilla from showing
 	 * up in the Nautilus history.
@@ -867,15 +858,15 @@ mozilla_location_callback (GtkMozEmbed *mozilla, gpointer user_data)
 		if (view->details->user_initiated_navigation) {
 			update_nautilus_uri (view, new_location_translated);
 		} else {
-			/* We still need to update our concept of the URI
-			 * so that our handling of relative URI's is correct.
-			 * This means a Nautilus-initiated reload will actually
-			 * fail to be communicated to mozilla as a special reload
-			 * event in view_load_location_callback.
-			 */
+			DEBUG_MSG (("=%s : Navigation not user initiated, reporting as redirect\n", __FUNCTION__));
+
+			DEBUG_MSG ((">nautilus_view_report_redirect (%s,%s)\n", view->details->uri, new_location_translated));
+
+			nautilus_view_report_redirect (view->details->nautilus_view, 
+				view->details->uri, new_location_translated, NULL, new_location_translated);
+
 			g_free (view->details->uri);
 			view->details->uri = g_strdup (new_location_translated);
-			DEBUG_MSG (("=%s : Navigation not user initiated, not reporting to nautilus\n", __FUNCTION__));
 		}
 	} else {
 		DEBUG_MSG (("=%s : URI's identical, ignoring request\n", __FUNCTION__));
@@ -981,7 +972,8 @@ mozilla_net_start_callback (GtkMozEmbed 	*mozilla,
 
 	g_assert (GTK_MOZ_EMBED (mozilla) == GTK_MOZ_EMBED (view->details->mozilla));
 
-	async_report_load_underway (view);
+	DEBUG_MSG ((">nautilus_view_report_load_underway\n"));
+	nautilus_view_report_load_underway (view->details->nautilus_view);
 
 	DEBUG_MSG (("-%s\n", __FUNCTION__));
 }
@@ -998,7 +990,8 @@ mozilla_net_stop_callback (GtkMozEmbed 	*mozilla,
 
 	g_assert (GTK_MOZ_EMBED (mozilla) == view->details->mozilla);
 
-	async_report_load_complete (view);
+	DEBUG_MSG ((">nautilus_view_report_load_complete\n"));
+	nautilus_view_report_load_complete (view->details->nautilus_view);
 
 	DEBUG_MSG (("-%s\n", __FUNCTION__));
 }
@@ -1030,7 +1023,7 @@ mozilla_link_message_callback (GtkMozEmbed *mozilla, gpointer user_data)
 
 	/* DEBUG_MSG (("=%s new link message '%s'\n", __FUNCTION__, translated_link_message)); */
 
-	async_report_status (view, translated_link_message);
+	nautilus_view_report_status (view->details->nautilus_view, translated_link_message);
 	g_free (translated_link_message);
 	g_free (link_message);
 
@@ -1058,11 +1051,14 @@ mozilla_progress_callback (GtkMozEmbed *mozilla,
 	 */
 
 	if (max_progress == -1 || max_progress == 0) {
-		async_report_load_progress (view, 0);
+		DEBUG_MSG ((">nautilus_view_report_load_progress %f\n", 0.0));
+		nautilus_view_report_load_progress (view->details->nautilus_view, 0);
 	} else if (max_progress < current_progress) {
-		async_report_load_progress (view, 1.0);
+		DEBUG_MSG ((">nautilus_view_report_load_progress %f\n", 1.0));
+		nautilus_view_report_load_progress (view->details->nautilus_view, 1.0);
 	} else {
-		async_report_load_progress (view, current_progress / max_progress);
+		DEBUG_MSG ((">nautilus_view_report_load_progress %f\n", (double)current_progress / (double)max_progress));
+		nautilus_view_report_load_progress (view->details->nautilus_view, (double)current_progress / (double)max_progress);
 	}
 }
 
@@ -1176,14 +1172,35 @@ mozilla_dom_mouse_click_callback (GtkMozEmbed *mozilla,
 			 * Right now, we report a load error.  But we
 			 * could tell ammonite to prompt for login
 			 */
-			async_report_load_failed (view);
+			DEBUG_MSG ((">nautilus_view_report_load_failed\n"));
+			nautilus_view_report_load_failed (view->details->nautilus_view);
 			ret = NS_DOM_EVENT_CONSUMED;
 		} else if (href[0] == '#') {
-			/* a navigation to an anchor within the same page, let it pass */
+			/* a navigation to an anchor within the same page */
 			view->details->user_initiated_navigation = TRUE;
-			ret = NS_DOM_EVENT_IGNORED;
+
+			/* bugzilla.mozilla.org 70311 -- anchor navigation
+			 * doesn't work on documents that have been streamed
+			 * into gtkmozembed.  So we do it outselves.
+			 */
+			if (should_mozilla_load_uri_directly (href_full)) {
+				DEBUG_MSG (("=%s : anchor navigation in normal uri, allowing navigate to continue\n", __FUNCTION__));
+				ret = NS_DOM_EVENT_IGNORED;
+			} else {
+				char *unescaped_anchor;
+
+				/* href+1 to skip the fragment identifier */
+				unescaped_anchor = gnome_vfs_unescape_string (href+1, NULL);
+
+				DEBUG_MSG (("=%s : anchor navigation in gnome-vfs uri, navigate by hand to anchor '%s'\n", __FUNCTION__, unescaped_anchor));
+
+				mozilla_navigate_to_anchor (view->details->mozilla, unescaped_anchor);
+				g_free (unescaped_anchor);
+				ret = NS_DOM_EVENT_CONSUMED;
+			}
 		} else if (0 == strncmp (href, "javascript:", strlen ("javascript:"))) {
 			/* This is a bullshit javascript uri, let it pass */
+			DEBUG_MSG (("=%s : javascript uri, allowing navigate to continue\n", __FUNCTION__));
 			ret = NS_DOM_EVENT_IGNORED;			
 		} else if (should_uri_navigate_bypass_nautilus (href_full)) {
 			view->details->user_initiated_navigation = TRUE;
@@ -1212,7 +1229,8 @@ mozilla_dom_mouse_click_callback (GtkMozEmbed *mozilla,
 		} else {
 			/* With some schemes, navigation needs to be funneled through nautilus. */
 			DEBUG_MSG (("=%s : funnelling navigation through nautilus\n", __FUNCTION__));
-			async_open_location (view, href_full);
+			DEBUG_MSG ((">nautilus_view_open_location_in_this_window '%s'", href_full));
+			nautilus_view_open_location_in_this_window (view->details->nautilus_view, href_full);
 
 			ret = NS_DOM_EVENT_CONSUMED;
 		}
@@ -1249,15 +1267,15 @@ vfs_open_callback (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer
 
 	if (result != GNOME_VFS_OK)
 	{
-		DEBUG_MSG ((">nautilus_view_report_load_failed\n"));
 		gtk_moz_embed_close_stream (view->details->mozilla);
 		/* NOTE: the view may go away after a call to report_load_failed */
+		DEBUG_MSG ((">nautilus_view_report_load_failed\n"));
 		nautilus_view_report_load_failed (view->details->nautilus_view);
 	} else {
 		if (view->details->vfs_read_buffer == NULL) {
 			view->details->vfs_read_buffer = g_malloc (VFS_READ_BUFFER_SIZE);
 		}
-		gtk_moz_embed_open_stream (view->details->mozilla, "file://", "text/html");
+		gtk_moz_embed_open_stream (view->details->mozilla, "file:///", "text/html");
 		gnome_vfs_async_read (handle, view->details->vfs_read_buffer, VFS_READ_BUFFER_SIZE, vfs_read_callback, view);
 	}
 	DEBUG_MSG (("-%s\n", __FUNCTION__));
@@ -1402,27 +1420,62 @@ navigate_mozilla_to_nautilus_uri (NautilusMozillaContentView     *view,
 			 	  const char			*nautilus_uri)
 {
 	char *mozilla_uri;
+	char *old_mozilla_uri;
+	char *old_nautilus_uri;
 
 	mozilla_uri = translate_uri_nautilus_to_mozilla (view, nautilus_uri);
 
 	cancel_pending_vfs_operation (view);
 
 	if (mozilla_uri == NULL) {
-		async_report_load_failed (view);
+		DEBUG_MSG ((">nautilus_view_report_load_failed\n"));
+		nautilus_view_report_load_failed (view->details->nautilus_view);
 		goto error;
-	} else if (should_mozilla_load_uri_directly (nautilus_uri)) {
-		if (view->details->uri != NULL && uris_identical (nautilus_uri, view->details->uri)) {
-			DEBUG_MSG (("=%s uri's identical, telling mozilla to reload\n", __FUNCTION__));
-			gtk_moz_embed_reload (view->details->mozilla,
-				GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE);
+	}
 
-		} else {
-			gtk_moz_embed_load_url (view->details->mozilla,
-						mozilla_uri);
-		}
+	if (!GTK_WIDGET_REALIZED (view->details->mozilla)) {
+
+		/* Doing certain things to gtkmozembed before
+		 * the widget has realized (specifically, opening
+		 * content streams) can cause crashes.  To avoid
+		 * this, we postpone all navigations
+		 * until the widget has realized (we believe
+		 * premature realization may cause other issues)
+		 */
+
+		DEBUG_MSG (("=%s: Postponing navigation request to widget realization\n", __FUNCTION__));
+		/* Note that view->details->uri is still set below */
 	} else {
-		DEBUG_MSG (("=%s loading URI via gnome-vfs\n", __FUNCTION__));
-		gnome_vfs_async_open (&(view->details->vfs_handle), nautilus_uri, GNOME_VFS_OPEN_READ, vfs_open_callback, view);
+		if (should_mozilla_load_uri_directly (nautilus_uri)) {
+
+			/* See if the current URI is the same as what mozilla already
+			 * has.  If so, issue a reload rather than a load.
+			 * We ask mozilla for it's uri rather than using view->details->uri because,
+			 * from time to time, our understanding of mozilla's URI can become inaccurate
+			 * (in particular, certain errors may cause embedded mozilla to not change
+			 * locations)
+			 */
+
+			old_mozilla_uri = gtk_moz_embed_get_location (view->details->mozilla);
+			old_nautilus_uri = translate_uri_mozilla_to_nautilus (view, old_mozilla_uri);
+
+			if (old_nautilus_uri != NULL && uris_identical (nautilus_uri, old_nautilus_uri)) {
+				DEBUG_MSG (("=%s uri's identical, telling mozilla to reload\n", __FUNCTION__));
+				gtk_moz_embed_reload (view->details->mozilla,
+					GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE);
+			} else {
+				gtk_moz_embed_load_url (view->details->mozilla,
+							mozilla_uri);
+			}
+
+			g_free (old_mozilla_uri);
+			old_mozilla_uri = NULL;
+			g_free (old_nautilus_uri);
+			old_nautilus_uri = NULL;
+		} else {
+			DEBUG_MSG (("=%s loading URI via gnome-vfs\n", __FUNCTION__));
+			gnome_vfs_async_open (&(view->details->vfs_handle), nautilus_uri, GNOME_VFS_OPEN_READ, vfs_open_callback, view);
+		}
 	}
 
 	g_free (view->details->uri);
@@ -1443,7 +1496,11 @@ update_nautilus_uri (NautilusMozillaContentView *view, const char *nautilus_uri)
 
 	DEBUG_MSG (("=%s current URI is now '%s'\n", __FUNCTION__, view->details->uri));
 
-	async_report_location_change (view, view->details->uri);
+	DEBUG_MSG ((">nautilus_view_report_location_change '%s'\n", nautilus_uri));
+	nautilus_view_report_location_change (view->details->nautilus_view,
+					      nautilus_uri,
+					      NULL,
+					      nautilus_uri);
 }
 
 /***********************************************************************************/
@@ -1972,7 +2029,7 @@ eazel_services_scheme_to_http (NautilusMozillaContentView	*view,
 }
 
 static char *
-eazel_services_scheme_from_http	(NautilusMozillaContentView	*view, 
+eazel_services_scheme_from_http	(NautilusMozillaContentView	*view,
 				 const char			*uri)
 {
 	AmmoniteError err;
@@ -1990,227 +2047,3 @@ eazel_services_scheme_from_http	(NautilusMozillaContentView	*view,
 }
 
 #endif /* EAZEL_SERVICES */
-
-
-/***********************************************************************************/
-/***********************************************************************************/
-
-/*
- * What's going on here?
- * 
- * Below are wrapper functions that delay outbound NautilusViewFrame callbacks
- * until we return to the GTK idle loop.  The problem is this:
- * 
- * o Any outbound ORBit call, including oneway calls, may result in a
- *   pending inbound call being dispatched
- * o Some inbound calls, particularly Bonobo::Control::realize, cannot safely
- *   be handled away from the event loop.  Indeed, if Bonobo::Control::realize
- *   gets called during a callback from Mozilla, the mozilla view will deadlock.
- *   
- * By postponing all outbound ORBit calls invoked as a result of mozilla callback
- * to the event loop, we eliminated the possibility that inbound calls can occur
- * during callbacks from Mozilla
- * 
- * Note: outbound ORBit calls that do not occur as a result of a mozilla callback
- * do not need to be so wrapped
- *
- * FIXME:  We want to move this type of thing into libnautilus
- */
-
-/*
- * Note that the "unref" in the calls below matches the "ref" in the async_
- * calls below
- */ 
-
-#define DISPATCH_NOARG_TMPL(FUNCNAME) \
-	static int /* GtkFunction */								\
-	dispatch_##FUNCNAME (gpointer data)							\
-	{											\
-		NautilusMozillaContentView *view;						\
-												\
-		view = NAUTILUS_MOZILLA_CONTENT_VIEW (data);					\
-												\
-		view->details->is_pending_##FUNCNAME = FALSE;					\
-												\
-		if (!GTK_OBJECT_DESTROYED (view)) {						\
-			DEBUG_MSG ((">>nautilus_view_" #FUNCNAME "\n")); 			\
-			nautilus_view_##FUNCNAME (view->details->nautilus_view); 		\
-		}										\
-		gtk_object_unref (GTK_OBJECT (view));						\
-												\
-		return 0;									\
-	}
-
-#define DISPATCH_STRING_TMPL(FUNCNAME, REALFUNCNAME, ARGNAME) \
-	static int /* GtkFunction */								\
-	dispatch_##FUNCNAME (gpointer data)							\
-	{											\
-		NautilusMozillaContentView *view;						\
-												\
-		view = NAUTILUS_MOZILLA_CONTENT_VIEW (data);					\
-												\
-		view->details->is_pending_##FUNCNAME = FALSE;					\
-												\
-		if (!GTK_OBJECT_DESTROYED (view)) {						\
-			DEBUG_MSG ((">>nautilus_view_" #REALFUNCNAME " '%s'\n", view->details->pending_##ARGNAME)); \
-			nautilus_view_##REALFUNCNAME (view->details->nautilus_view, view->details->pending_##ARGNAME); \
-			g_free (view->details->pending_##ARGNAME);				\
-			view->details->pending_##ARGNAME = NULL;				\
-		}										\
-		gtk_object_unref (GTK_OBJECT (view));						\
-												\
-		return 0;									\
-	}
-
-#define DISPATCH_STRING_NO_DEBUG_TMPL(FUNCNAME, REALFUNCNAME, ARGNAME) \
-	static int /* GtkFunction */								\
-	dispatch_##FUNCNAME (gpointer data)							\
-	{											\
-		NautilusMozillaContentView *view;						\
-												\
-		view = NAUTILUS_MOZILLA_CONTENT_VIEW (data);					\
-												\
-		view->details->is_pending_##FUNCNAME = FALSE;					\
-												\
-		if (!GTK_OBJECT_DESTROYED (view)) {						\
-			nautilus_view_##REALFUNCNAME (view->details->nautilus_view, view->details->pending_##ARGNAME); \
-			g_free (view->details->pending_##ARGNAME);				\
-			view->details->pending_##ARGNAME = NULL;				\
-		}										\
-		gtk_object_unref (GTK_OBJECT (view));						\
-												\
-		return 0;									\
-	}
-
-DISPATCH_NOARG_TMPL (report_load_failed)
-
-DISPATCH_NOARG_TMPL (report_load_underway)
-
-DISPATCH_NOARG_TMPL (report_load_complete)
-
-DISPATCH_STRING_TMPL (set_title, set_title, title)
-
-DISPATCH_STRING_NO_DEBUG_TMPL (report_status, report_status, link_message)
-
-DISPATCH_STRING_TMPL (open_location, open_location_in_this_window, open_uri)
-
-static int /* GtkFunction */
-dispatch_report_location_change (gpointer data)
-{
-	NautilusMozillaContentView *view;
-
-	view = NAUTILUS_MOZILLA_CONTENT_VIEW (data);
-
-	view->details->is_pending_report_location_change = FALSE;
-
-	if (!GTK_OBJECT_DESTROYED (view)) {
-		nautilus_view_report_location_change (view->details->nautilus_view, view->details->pending_report_uri, NULL, view->details->pending_report_uri);
-		DEBUG_MSG ((">>nautilus_view_report_location_change\n"));
-
-		g_free (view->details->pending_report_uri);
-		view->details->pending_report_uri = NULL;
-	}
-
-	gtk_object_unref (GTK_OBJECT (view));
-
-	return 0;
-}
-
-static int /* GtkFunction */
-dispatch_report_load_progress (gpointer data)
-{
-	NautilusMozillaContentView *view;
-
-	view = NAUTILUS_MOZILLA_CONTENT_VIEW (data);
-
-	view->details->is_pending_report_load_progress = FALSE;
-
-	if (!GTK_OBJECT_DESTROYED (view)) {
-		DEBUG_MSG ((">>nautilus_view_report_load_progress %f\n", 
-			view->details->pending_load_progress));
-		nautilus_view_report_load_progress (view->details->nautilus_view, view->details->pending_load_progress);
-	}
-
-	gtk_object_unref (GTK_OBJECT (view));
-
-	return 0;
-}
-
-/*
- * These async_ functions delay CORBA calls by registering a gtk_timeout
- * with the callbacks being the dispatch functions above.
- * 
- * Note that a gtk reference is kept across the timeout callbacks
- * to ensure the object is still valid
- */
-
-#define ASYNC_NOARG_WRAPPER_TMPL(FUNCNAME)  \
-	static void									\
-	async_##FUNCNAME (NautilusMozillaContentView *view)	\
-	{										\
-		DEBUG_MSG (("~" #FUNCNAME "\n")); 					\
-											\
-		if (!view->details->is_pending_##FUNCNAME) {				\
-			view->details->is_pending_##FUNCNAME = TRUE;			\
-			gtk_object_ref (GTK_OBJECT (view));				\
-			gtk_timeout_add (0, dispatch_##FUNCNAME, view);			\
-		}									\
-	}
-
-#define ASYNC_STRING_WRAPPER_TMPL(FUNCNAME, ARGNAME)  \
-	static void									\
-	async_##FUNCNAME (NautilusMozillaContentView *view, const char *ARGNAME)	\
-	{										\
-		g_free (view->details->pending_##ARGNAME);				\
-		view->details->pending_##ARGNAME = g_strdup (ARGNAME);			\
-											\
-		DEBUG_MSG (("~" #FUNCNAME " '%s'\n", view->details->pending_##ARGNAME)); \
-											\
-		if (!view->details->is_pending_##FUNCNAME) {				\
-			view->details->is_pending_##FUNCNAME = TRUE;			\
-			gtk_object_ref (GTK_OBJECT (view));				\
-			gtk_timeout_add (0, dispatch_##FUNCNAME, view);			\
-		}									\
-	}
-
-#define ASYNC_STRING_WRAPPER_NO_DEBUG_TMPL(FUNCNAME, ARGNAME)  \
-	static void									\
-	async_##FUNCNAME (NautilusMozillaContentView *view, const char *ARGNAME)	\
-	{										\
-		g_free (view->details->pending_##ARGNAME);				\
-		view->details->pending_##ARGNAME = g_strdup (ARGNAME);			\
-											\
-		if (!view->details->is_pending_##FUNCNAME) {				\
-			view->details->is_pending_##FUNCNAME = TRUE;			\
-			gtk_object_ref (GTK_OBJECT (view));				\
-			gtk_timeout_add (0, dispatch_##FUNCNAME, view);			\
-		}									\
-	}
-
-ASYNC_NOARG_WRAPPER_TMPL (report_load_failed)
-
-ASYNC_NOARG_WRAPPER_TMPL (report_load_underway)
-
-ASYNC_NOARG_WRAPPER_TMPL (report_load_complete)
-
-ASYNC_STRING_WRAPPER_TMPL (set_title, title)
-
-ASYNC_STRING_WRAPPER_TMPL (report_location_change, report_uri)
-
-ASYNC_STRING_WRAPPER_NO_DEBUG_TMPL (report_status, link_message)
-
-ASYNC_STRING_WRAPPER_TMPL (open_location, open_uri)
-
-static void
-async_report_load_progress (NautilusMozillaContentView *view, double load_progress)
-{
-	view->details->pending_load_progress = load_progress;
-	
-	if (!view->details->is_pending_report_load_progress) {
-		DEBUG_MSG (("~report_load_progress %f\n", 
-			view->details->pending_load_progress));
-		view->details->is_pending_report_load_progress = TRUE;
-		gtk_object_ref (GTK_OBJECT (view));
-		gtk_timeout_add (0, dispatch_report_load_progress, view);
-	}
-}
