@@ -46,8 +46,6 @@
 #include "nautilus-gdk-extensions.h"
 #include "nautilus-gtk-extensions.h"
 #include "nautilus-xml-extensions.h"
-#include "nautilus-image.h"
-#include "nautilus-label.h"
 #include "nautilus-string.h"
 
 typedef enum {
@@ -59,15 +57,14 @@ struct NautilusCustomizationData {
 	char *customization_name;
 	CustomizationReadingMode reading_mode;
 
-	GnomeVFSDirectoryList *public_file_list;	
-	GnomeVFSDirectoryList *private_file_list;
-	GnomeVFSDirectoryList *current_file_list;
+	GList *public_file_list;	
+	GList *private_file_list;
+	GList *current_file_list;
 
 	GHashTable *name_map_hash;
 	
 	GdkPixbuf *pattern_frame;
 
-	gboolean started_reading_current_file_list;
 	gboolean private_data_was_displayed;
 	gboolean data_is_for_a_menu;
 	int maximum_icon_height;
@@ -140,7 +137,6 @@ nautilus_customization_data_new (const char *customization_name,
 	}
 
 	
-	data->started_reading_current_file_list = FALSE;
 	data->private_data_was_displayed = FALSE;
 	data->data_is_for_a_menu = data_is_for_a_menu;
 	data->customization_name = g_strdup (customization_name);
@@ -157,54 +153,56 @@ nautilus_customization_data_new (const char *customization_name,
 GnomeVFSResult
 nautilus_customization_data_get_next_element_for_display (NautilusCustomizationData *data,
 							  char **emblem_name,
-							  GtkWidget **pixmap_widget,
-							  GtkWidget **label)
+							  GdkPixbuf **pixbuf_out,
+							  char **label_out)
 {
 	GnomeVFSFileInfo *current_file_info;
 
-	char *image_file_name, *filtered_name, *truncated_name;
+	char *image_file_name, *filtered_name;
 	GdkPixbuf *pixbuf;
 	GdkPixbuf *orig_pixbuf;
-
-	if (!data->started_reading_current_file_list) {
-		current_file_info = gnome_vfs_directory_list_first (data->current_file_list);
-		data->started_reading_current_file_list = TRUE;
-	}
-	else {
-		current_file_info = gnome_vfs_directory_list_next (data->current_file_list);
-	}
-	if (current_file_info == NULL) {
+	
+	g_return_val_if_fail (data != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (emblem_name != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (pixbuf_out != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (label_out != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	
+	if (data->current_file_list == NULL) {
 		if (data->reading_mode == READ_PUBLIC_CUSTOMIZATIONS) {
 			if (data->private_file_list == NULL) {
 				return GNOME_VFS_ERROR_EOF;
 			}
 			data->reading_mode = READ_PRIVATE_CUSTOMIZATIONS;
 			data->current_file_list = data->private_file_list;
-			data->started_reading_current_file_list = FALSE;
 			return nautilus_customization_data_get_next_element_for_display (data,
 											 emblem_name,
-											 pixmap_widget,
-											 label);
+											 pixbuf_out,
+											 label_out);
 		}
 		else {
-				return GNOME_VFS_ERROR_EOF;
+			return GNOME_VFS_ERROR_EOF;
 		}
 	}
-
 	
+	
+	current_file_info = data->current_file_list->data;
+	data->current_file_list = data->current_file_list->next;
+
+	g_assert (current_file_info != NULL);
+
 	if (!nautilus_istr_has_prefix (current_file_info->mime_type, "image/") ||
 	    nautilus_istr_has_prefix (current_file_info->name, ".")) {
 		
 		return nautilus_customization_data_get_next_element_for_display (data,
 										 emblem_name,
-										 pixmap_widget,
-										 label);
+										 pixbuf_out,
+										 label_out);
 	}
 
 	image_file_name = get_file_path_for_mode (data,
 						  current_file_info->name);
-	orig_pixbuf = gdk_pixbuf_new_from_file(image_file_name);
-	g_free(image_file_name);
+	orig_pixbuf = gdk_pixbuf_new_from_file (image_file_name);
+	g_free (image_file_name);
 
 	*emblem_name = g_strdup (current_file_info->name);
 	
@@ -217,9 +215,7 @@ nautilus_customization_data_get_next_element_for_display (NautilusCustomizationD
 		gdk_pixbuf_unref (orig_pixbuf);
 	}
 	
-	*pixmap_widget = nautilus_image_new (NULL);
-	nautilus_image_set_pixbuf (NAUTILUS_IMAGE (*pixmap_widget), pixbuf);
-	gdk_pixbuf_unref (pixbuf);
+	*pixbuf_out = pixbuf;
 	
 	filtered_name = format_name_for_display (data, current_file_info->name);
 	/* If the data is for a menu,
@@ -227,12 +223,10 @@ nautilus_customization_data_get_next_element_for_display (NautilusCustomizationD
 	   label because anti-aliased text doesn't look right
 	   in menus */
 	if (data->data_is_for_a_menu) {
-		truncated_name = nautilus_truncate_text_for_menu_item (filtered_name);
-		*label = gtk_label_new (truncated_name);
-		g_free (truncated_name);
+		*label_out = nautilus_truncate_text_for_menu_item (filtered_name);
 	}
 	else {
-		*label = nautilus_label_new (filtered_name);
+		*label_out = g_strdup (filtered_name);
 	}
 	
 	g_free (filtered_name);
@@ -259,12 +253,9 @@ nautilus_customization_data_destroy (NautilusCustomizationData *data)
 		gdk_pixbuf_unref (data->pattern_frame);
 	}
 
-	if (data->public_file_list != NULL) {
-		gnome_vfs_directory_list_destroy (data->public_file_list);
-	}
-	if (data->private_file_list != NULL) {
-		gnome_vfs_directory_list_destroy (data->private_file_list);
-	}
+	gnome_vfs_file_info_list_free (data->public_file_list);
+	gnome_vfs_file_info_list_free (data->private_file_list);
+
 	if (data->name_map_hash != NULL) {
 		nautilus_g_hash_table_destroy_deep (data->name_map_hash);	
 	}
