@@ -85,8 +85,8 @@ finish_button_activation (gpointer data)
 		gtk_button_released (button);
 	}
 
-	/* this was ref'd in nautilus_gtk_button_auto_click */
-	gtk_object_unref (GTK_OBJECT(button));
+	/* This was ref'd in nautilus_gtk_button_auto_click */
+	gtk_object_unref (GTK_OBJECT (button));
 
 	return FALSE;	
 }
@@ -121,7 +121,7 @@ nautilus_gtk_button_auto_click (GtkButton *button)
 	 */
 
 	/* This is unref'ed in finish_button_activation */
-	gtk_object_ref (GTK_OBJECT(button));
+	gtk_object_ref (GTK_OBJECT (button));
 
 	g_timeout_add (BUTTON_AUTO_HIGHLIGHT_MILLISECONDS, 
 		       finish_button_activation, button);
@@ -299,10 +299,11 @@ guint nautilus_gtk_signal_connect_free_data_custom (GtkObject *object,
  * @data: the user data associated with the function. g_free() will be called on
  * this user data when the signal is disconnected.
  **/
-guint nautilus_gtk_signal_connect_free_data (GtkObject *object,
-				  	     const gchar *name,
-				  	     GtkSignalFunc func,
-				  	     gpointer data)
+guint
+nautilus_gtk_signal_connect_free_data (GtkObject *object,
+				       const gchar *name,
+				       GtkSignalFunc func,
+				       gpointer data)
 {
 	return nautilus_gtk_signal_connect_free_data_custom
 		(object, name, func, data, g_free);
@@ -1021,16 +1022,17 @@ nautilus_gtk_marshal_POINTER__POINTER_POINTER_POINTER (GtkObject *object,
 }
 
 void
-nautilus_gtk_marshal_NONE__POINTER_POINTER_POINTER (GtkObject *object,
-						    GtkSignalFunc func,
-						    gpointer func_data,
-						    GtkArg *args)
+nautilus_gtk_marshal_NONE__POINTER_POINTER_POINTER_POINTER (GtkObject *object,
+							    GtkSignalFunc func,
+							    gpointer func_data,
+							    GtkArg *args)
 {
-	(* (void (*)(GtkObject *, gpointer, gpointer, gpointer, gpointer)) func)
+	(* (void (*)(GtkObject *, gpointer, gpointer, gpointer, gpointer, gpointer)) func)
 		(object,
 		 GTK_VALUE_POINTER (args[0]),
 		 GTK_VALUE_POINTER (args[1]),
 		 GTK_VALUE_POINTER (args[2]),
+		 GTK_VALUE_POINTER (args[3]),
 		 func_data);
 }
 
@@ -1182,6 +1184,25 @@ nautilus_gtk_widget_set_font (GtkWidget *widget, GdkFont *font)
 	gtk_style_unref (new_style);
 }
 
+/**
+ * nautilus_gtk_widget_set_shown
+ *
+ * Show or hide a widget.
+ * @widget: The widget.
+ * @shown: Boolean value indicating whether the widget should be shown or hidden.
+ **/
+void
+nautilus_gtk_widget_set_shown (GtkWidget *widget, gboolean shown)
+{
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+
+	if (shown) {
+		gtk_widget_show (widget);
+	} else {
+		gtk_widget_hide (widget);
+	}
+}
+
 /**
  * nautilus_gtk_widget_set_font_by_name
  *
