@@ -86,6 +86,8 @@ void              nautilus_gtk_object_list_free                        (GList
 GList *           nautilus_gtk_object_list_copy                        (GList                    *list);
 
 /* GtkWidget */
+void              nautilus_gtk_widget_set_shown                        (GtkWidget                *widget,
+									gboolean                 shown);
 gboolean          nautilus_point_in_allocation                         (const GtkAllocation      *allocation,
 									int                       x,
 									int                       y);
@@ -175,20 +177,21 @@ GList		  *nautilus_get_window_list_ordered_front_to_back      (void);
 
 /* marshals */
 
-#define nautilus_gtk_marshal_NONE__BOXED_BOXED gtk_marshal_NONE__POINTER_POINTER
-#define nautilus_gtk_marshal_NONE__POINTER_STRING_STRING nautilus_gtk_marshal_NONE__POINTER_POINTER_POINTER
-#define nautilus_gtk_marshal_NONE__STRING_POINTER gtk_marshal_NONE__POINTER_POINTER
-#define nautilus_gtk_marshal_NONE__STRING_POINTER_STRING gtk_marshal_NONE__POINTER_POINTER_POINTER
 #define nautilus_gtk_marshal_BOOL__POINTER_POINTER nautilus_gtk_marshal_INT__POINTER_POINTER
 #define nautilus_gtk_marshal_INT__POINTER_STRING nautilus_gtk_marshal_INT__POINTER_POINTER
-#define nautilus_gtk_marshal_POINTER__POINTER_STRING_POINTER nautilus_gtk_marshal_POINTER__POINTER_POINTER_POINTER
+#define nautilus_gtk_marshal_NONE__BOXED_BOXED gtk_marshal_NONE__POINTER_POINTER
+#define nautilus_gtk_marshal_NONE__POINTER_STRING_STRING gtk_marshal_NONE__POINTER_POINTER_POINTER
+#define nautilus_gtk_marshal_NONE__STRING_POINTER gtk_marshal_NONE__POINTER_POINTER
+#define nautilus_gtk_marshal_NONE__STRING_POINTER_STRING gtk_marshal_NONE__POINTER_POINTER_POINTER
+#define nautilus_gtk_marshal_NONE__STRING_STRING_POINTER_STRING nautilus_gtk_marshal_NONE__POINTER_POINTER_POINTER_POINTER
 #define nautilus_gtk_marshal_POINTER__POINTER_INT_INT_STRING_POINTER nautilus_gtk_marshal_POINTER__POINTER_INT_INT_POINTER_POINTER
+#define nautilus_gtk_marshal_POINTER__POINTER_STRING_POINTER nautilus_gtk_marshal_POINTER__POINTER_POINTER_POINTER
 #define nautilus_gtk_marshal_STRING__NONE nautilus_gtk_marshal_POINTER__NONE
 #define nautilus_gtk_marshal_STRING__POINTER nautilus_gtk_marshal_POINTER__POINTER
 #define nautilus_gtk_marshal_STRING__POINTER_POINTER nautilus_gtk_marshal_POINTER__POINTER_POINTER
-#define nautilus_gtk_marshal_STRING__POINTER_STRING nautilus_gtk_marshal_POINTER__POINTER_POINTER
 #define nautilus_gtk_marshal_STRING__POINTER_POINTER_POINTER nautilus_gtk_marshal_POINTER__POINTER_POINTER_POINTER
 #define nautilus_gtk_marshal_STRING__POINTER_POINTER_STRING nautilus_gtk_marshal_POINTER__POINTER_POINTER_POINTER
+#define nautilus_gtk_marshal_STRING__POINTER_STRING nautilus_gtk_marshal_POINTER__POINTER_POINTER
 
 void nautilus_gtk_marshal_INT__NONE                                             (GtkObject     *object,
 										 GtkSignalFunc  func,
@@ -226,7 +229,7 @@ void nautilus_gtk_marshal_NONE__INT_POINTER_INT_INT_UINT
 										 GtkSignalFunc  func,
 										 gpointer       func_data,
 										 GtkArg        *args);
-void nautilus_gtk_marshal_NONE__POINTER_POINTER_POINTER                         (GtkObject     *object,
+void nautilus_gtk_marshal_NONE__POINTER_POINTER_POINTER_POINTER                 (GtkObject     *object,
 										 GtkSignalFunc  func,
 										 gpointer       func_data,
 										 GtkArg        *args);
