@@ -26,6 +26,7 @@
 #include <config.h>
 #include "nautilus-preferences-pane.h"
 #include "nautilus-gtk-macros.h"
+#include "nautilus-gtk-extensions.h"
 
 #include <gtk/gtkhbox.h>
 
@@ -90,14 +91,10 @@ nautilus_preferences_pane_destroy (GtkObject* object)
  * NautilusPreferencesPane public methods
  */
 GtkWidget *
-nautilus_preferences_pane_new (const gchar *pane_title,
-			       const gchar *pane_description)
+nautilus_preferences_pane_new (void)
 {
 	NautilusPreferencesPane *preferences_pane;
 
-	g_return_val_if_fail (pane_title != NULL, NULL);
-	g_return_val_if_fail (pane_description != NULL, NULL);
-
 	preferences_pane = NAUTILUS_PREFERENCES_PANE
 		(gtk_widget_new (nautilus_preferences_pane_get_type (), NULL));
 
@@ -188,14 +185,9 @@ nautilus_preferences_pane_update (NautilusPreferencesPane *preferences_pane)
 
 	for (iterator = preferences_pane->details->groups; iterator != NULL; iterator = iterator->next) {
 		NautilusPreferencesGroup *group = NAUTILUS_PREFERENCES_GROUP (iterator->data);
-
 		nautilus_preferences_group_update (group);
-		
-		if (nautilus_preferences_group_get_num_visible_items (group) == 0) {
-			gtk_widget_hide (GTK_WIDGET (group));
-		} else {
-			gtk_widget_show (GTK_WIDGET (group));
-		}
+		nautilus_gtk_widget_set_shown (GTK_WIDGET (group),
+					       nautilus_preferences_group_get_num_visible_items (group) > 0);
 	}
 }
 
