@@ -47,6 +47,7 @@
 #include <libnautilus-extensions/nautilus-global-preferences.h>
 #include <libnautilus-extensions/nautilus-lib-self-check-functions.h>
 #include <libnautilus-extensions/nautilus-self-checks.h>
+#include <libnautilus-extensions/nautilus-directory-metafile.h>
 #include <liboaf/liboaf.h>
 #include <popt.h>
 #include <stdlib.h>
@@ -150,10 +151,6 @@ main (int argc, char *argv[])
 		  N_("Quit Nautilus."), NULL },
 		{ "restart", '\0', POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN, &restart_shell, 0,
 		  N_("Restart Nautilus."), NULL },
-		/* FIXME bugzilla.eazel.com 5228: The help for these OAF options
-		 * should be in a separate section.
-		 */
-		{ NULL, '\0', POPT_ARG_INCLUDE_TABLE, &oaf_popt_options, 0, NULL, NULL },
 		{ NULL, '\0', 0, NULL, 0, NULL, NULL }
 	};
 
@@ -195,6 +192,11 @@ main (int argc, char *argv[])
 	no_default_window = FALSE;
 	perform_self_check = FALSE;
 	restart_shell = FALSE;
+
+	gnomelib_register_popt_table (oaf_popt_options, 
+				      oaf_get_popt_table_name ());
+	orb = oaf_init (argc, argv);
+
         gnome_init_with_popt_table ("nautilus", VERSION,
 				    argc, argv, options, 0,
 				    &popt_context);
@@ -227,7 +229,7 @@ main (int argc, char *argv[])
 	LIBXML_TEST_VERSION
 	g_atexit (xmlCleanupParser);
 	g_thread_init (NULL);
-	orb = oaf_init (argc, argv);
+
 	if (g_getenv ("NAUTILUS_ENABLE_TEST_COMPONENTS") != NULL) {
 		oaf_set_test_components_enabled (TRUE);
 	}
@@ -247,6 +249,7 @@ main (int argc, char *argv[])
 	if (perform_self_check) {
 #ifndef NAUTILUS_OMIT_SELF_CHECK
 		/* Run the checks for nautilus and libnautilus (each twice). */
+		nautilus_directory_use_self_contained_metafile_factory ();
 		nautilus_run_self_checks ();
 		nautilus_run_lib_self_checks ();
 		nautilus_exit_if_self_checks_failed ();
@@ -259,7 +262,9 @@ main (int argc, char *argv[])
 		application = nautilus_application_new ();
 		nautilus_application_startup
 			(application,
-			 kill_shell, restart_shell, start_desktop, no_default_window, TRUE, geometry,
+			 kill_shell, restart_shell, start_desktop, no_default_window,
+			 !(kill_shell || restart_shell),
+			 geometry,
 			 args);
 		if (is_event_loop_needed ()) {
 			bonobo_main ();
