@@ -110,7 +110,7 @@ static void
 nautilus_switchable_navigation_bar_destroy (GtkObject *object)
 {
 	g_free (NAUTILUS_SWITCHABLE_NAVIGATION_BAR (object)->details);
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 GtkWidget *
