@@ -536,6 +536,34 @@ cancel_viewed_file_changed_callback (NautilusWindow *window)
         }
 }
 
+static void
+update_history (NautilusWindow *window,
+                NautilusLocationChangeType type,
+                const char *new_location)
+{
+        switch (type) {
+        case NAUTILUS_LOCATION_CHANGE_STANDARD:
+                nautilus_window_add_current_location_to_history_list (window);
+                handle_go_elsewhere (window, new_location);
+                return;
+        case NAUTILUS_LOCATION_CHANGE_RELOAD:
+                /* for reload there is no work to do */
+                return;
+        case NAUTILUS_LOCATION_CHANGE_BACK:
+                nautilus_window_add_current_location_to_history_list (window);
+                handle_go_back (window, new_location);
+                return;
+        case NAUTILUS_LOCATION_CHANGE_FORWARD:
+                nautilus_window_add_current_location_to_history_list (window);
+                handle_go_forward (window, new_location);
+                return;
+        case NAUTILUS_LOCATION_CHANGE_REDIRECT:
+                /* for the redirect case, the caller can do the updating */
+                return;
+        }
+	g_return_if_fail (FALSE);
+}
+
 /* Handle the changes for the NautilusWindow itself. */
 static void
 update_for_new_location (NautilusWindow *window)
@@ -546,22 +574,8 @@ update_for_new_location (NautilusWindow *window)
         new_location = window->details->pending_location;
         window->details->pending_location = NULL;
         
-        /* Maintain history lists. */
-        if (window->details->location_change_type != NAUTILUS_LOCATION_CHANGE_RELOAD) {
-                /* Always add new location to history list. */
-                nautilus_window_add_current_location_to_history_list (window);
+        update_history (window, window->details->location_change_type, new_location);
                 
-                /* Update back and forward list. */
-                if (window->details->location_change_type == NAUTILUS_LOCATION_CHANGE_BACK) {
-                        handle_go_back (window, new_location);
-                } else if (window->details->location_change_type == NAUTILUS_LOCATION_CHANGE_FORWARD) {
-                        handle_go_forward (window, new_location);
-                } else {
-                        g_assert (window->details->location_change_type == NAUTILUS_LOCATION_CHANGE_STANDARD);
-                        handle_go_elsewhere (window, new_location);
-                }
-        }
-        
         /* Set the new location. */
         g_free (window->details->location);
         window->details->location = new_location;
@@ -1043,18 +1057,16 @@ free_location_change (NautilusWindow *window)
         g_free (window->details->pending_location);
         window->details->pending_location = NULL;
 
-        if (window->details->pending_location_as_directory != NULL) {
-                nautilus_directory_file_monitor_remove (window->details->pending_location_as_directory,
-                                                        window);
-                nautilus_directory_unref (window->details->pending_location_as_directory);
-                window->details->pending_location_as_directory = NULL;
-        }
-
+        /* Important to do this first, because destroying the
+         * pending_location_as_directory can cause additional I/O to
+         * complete, which might cause the initial view callback to be
+         * called.
+         */
         if (window->details->determine_view_handle != NULL) {
                 nautilus_determine_initial_view_cancel (window->details->determine_view_handle);
                 window->details->determine_view_handle = NULL;
         }
-        
+
         if (window->new_content_view != NULL) {
         	if (window->new_content_view != window->content_view) {
                         disconnect_view (window, window->new_content_view);
@@ -1063,6 +1075,13 @@ free_location_change (NautilusWindow *window)
         	gtk_object_unref (GTK_OBJECT (window->new_content_view));
                 window->new_content_view = NULL;
         }
+        
+        if (window->details->pending_location_as_directory != NULL) {
+                nautilus_directory_file_monitor_remove (window->details->pending_location_as_directory,
+                                                        window);
+                nautilus_directory_unref (window->details->pending_location_as_directory);
+                window->details->pending_location_as_directory = NULL;
+        }
 }
 
 static void
@@ -1803,7 +1822,7 @@ report_location_change_callback (NautilusViewFrame *view,
         g_assert (NAUTILUS_IS_WINDOW (window));
 
         if (view != window->content_view) {
-                /* Do we need to do anything in this case. */
+                /* Do we need to do anything in this case? */
                 return;
         }
 
@@ -1814,12 +1833,53 @@ report_location_change_callback (NautilusViewFrame *view,
                                         selection,
                                         view);
         
-        /* Setting the change type to reload here is a hack. */
         window->details->location_change_type = NAUTILUS_LOCATION_CHANGE_STANDARD;
         window->details->pending_location = g_strdup (location);
         update_for_new_location (window);
 }
 
+static void
+report_redirect_callback (NautilusViewFrame *view,
+                          const char *from_location,
+                          const char *to_location,
+                          GList *selection,
+                          const char *title,
+                          NautilusWindow *window)
+{
+        const char *existing_location;
+
+        g_assert (NAUTILUS_IS_WINDOW (window));
+
+        if (view != window->content_view) {
+                /* Do we need to do anything in this case? */
+                return;
+        }
+
+        /* Ignore redirect if we aren't already at "from_location". */
+        existing_location = window->details->pending_location;
+        if (existing_location == NULL) {
+                existing_location = window->details->location;
+        }
+        if (existing_location == NULL
+            || !nautilus_uris_match (existing_location, from_location)) {
+                return;
+        }
+
+        end_location_change (window);
+
+        load_new_location_in_all_views (window,
+                                        to_location,
+                                        selection,
+                                        view);
+
+        nautilus_remove_from_history_list_no_notify (from_location);
+        nautilus_window_add_current_location_to_history_list (window);
+        
+        window->details->location_change_type = NAUTILUS_LOCATION_CHANGE_REDIRECT;
+        window->details->pending_location = g_strdup (to_location);
+        update_for_new_location (window);
+}
+
 static void
 title_changed_callback (NautilusViewFrame *view,
                         NautilusWindow *window)
@@ -1865,6 +1925,7 @@ view_loaded_callback (NautilusViewFrame *view,
 	macro (open_location_in_this_window)		\
 	macro (open_location_prefer_existing_window)	\
 	macro (report_location_change)			\
+	macro (report_redirect)				\
 	macro (title_changed)				\
 	macro (view_loaded)				\
 	macro (zoom_level_changed)			\
@@ -1951,9 +2012,16 @@ nautilus_window_back_or_forward (NautilusWindow *window, gboolean back, guint di
 void
 nautilus_window_reload (NautilusWindow *window)
 {
+	char *location;
+	
         g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 
+	/* window->details->location can be free'd during the processing
+	 * of begin_location_change, so make a copy
+	 */
+	location = g_strdup (window->details->location);
 	begin_location_change
-		(window, window->details->location,
+		(window, location,
 		 NAUTILUS_LOCATION_CHANGE_RELOAD, 0);
+	g_free (location);
 }
